% January 2020
% Copyright CNES TESA
%
%        MAIN CODE
% 
% Computation of SLA spectra using several methods:
%   1-Periodogram (Tuckey window)
%   2-ARWARP (see paper)
% Slope Estimation is done on the corresponding spectra through :
%   1 - Linear regression
%   2 - Model fitting (see paper)
% Case of simulated SLA generation: computation of CRB
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 
% Necessary functions (the comments in these functions may be in French)
%   generation_signal_bruit_simule_SLA
%   estimePenteBruit
%   estime_noise_gamma
%   calculARWARP 
%   get_index_norm
%   estime_pre_process
%   slope_plus_noise_gamma
%   generation_signal_simule_SLA
%   crb_whittle_slope_plus_noise

clear all; close all;

% Computation on real or simulated SLA signals
simuSLA = input('Computation on real (0) or simulated (1) SLA signals? ');

% simuSLA = 1; % = 1 computation on simulated SLA signals
% simuSLA = 0; % = 0 computation on real SLA signals
%


if simuSLA == 1,

    % Change parameters if required
    % structure Simu en entree de generation signal

    Simu.Slope = 3; % slope

    Simu.NSamples = 3000; % number of samples per segment
    Simu.NSegments = 10;  % number of segments
    Simu.NIters = 1; % should be = 1, internal parameter
    Simu.Dist = 0.319; % sampling distance in km (Sentinel-3)
    Simu.repexdetrend = 0; % could be used to add a tendency
    Simu.FMin = 0.001*1/Simu.Dist; % in km-1, frequency corresponding to the beginning of the slope (f1 in the paper)
    % 0.001 corresponds to f1 in normalized frequency
    Simu.Fcross = 0.01*1/Simu.Dist; % in km-1, frequency corresponding to the beginning of the noise floor
    Simu.NoiseLevel = 0.003; % noise level 
    % values in Simu.FMin, Simu.Fcross and Simu.NoiseLevel have been chosen
    % based on real signals (Sentinel 3)
    
    disp('Simulated SLA parameters');
    disp([' Slope = ' num2str(Simu.Slope)]);
    disp([' Number of segments = ' num2str(Simu.NSegments)]);
    disp([' Number of samples of each segment = ' num2str(Simu.NSamples)]);
    nom = ['Simulated SLA (slope = ' num2str(Simu.Slope) ')'];

    nbPts = Simu.NSamples; % number of samples per segment
    nbSeg = Simu.NSegments; % number of segments
    Fe = 1/Simu.Dist; % sampling frequency
    N = nbSeg*nbPts; % total number of samples 

    % call generation signal
    [nbSeg,nbPts,vSLA,F_true,Sx_true] = generation_signal_bruit_simule_SLA(Simu);
    gamma_dB = 10*( log10(max(Sx_true))- log10(min(Sx_true)) );
    
    % CRB 
    gamma = (max(Sx_true))/(min(Sx_true)); %10^(gamma_dB/10);
    alpha = Simu.Slope;
    sigma2 = Simu.NoiseLevel; %10^(sigma2_dB/10); % does not depend on sigma2
    fnorm_begin_slope = Simu.FMin/Fe ; %normalized frequency
    
    [CRB_sigma2_known,CRB_sigma2_unknown,CRB_slope_sigma2_known,CRB_slope_sigma2_unknown] = crb_whittle_slope_plus_noise(gamma,alpha,sigma2,fnorm_begin_slope);
    disp(' ********************** ');
    disp(['CRB = ' num2str(CRB_slope_sigma2_unknown / nbPts) ]);
    disp(' ********************** ');
    CRB = CRB_slope_sigma2_unknown / nbPts;

else
    % SLA signal file
    [nom,chemin] = uigetfile('*.nc','SLA file name'); % 

    SLA = ncread([chemin nom],'Matrix');
    dist = ncread([chemin nom],'Step_N1'); % sampling frequency in km
        
    Fe = 1/dist; % sampling frequency
    nbPts = size(SLA,1); % number of samples per segment
    nbSeg = size(SLA,2); % number of segments
    N = nbSeg*nbPts; % total number of samples (necessary to reshape)

    % SLA matrix transformed in vector (to be coherent with simulated SLA
    % signals)
    vSLA = reshape(SLA,1,N);
end

%%%% signal display
figure('Name',nom); 
    plot(vSLA); title([nom ' ' num2str(nbSeg) ' segments of ' num2str(nbPts) ' samples' ],'interpreter','none');
    v = axis; hold on;
    M =nbPts;
    for k=1:round(N/M),
        plot([M*k M*k],[v(3) v(4)],'g--');
    end

%%%%%%%%%%%%%% ANALYSIS PARAMETERS %%%%%%%%%%%%%%%%%%
% Spectral Analysis by Periodogram, Tukey 10%, zero padding
fen = tukeywin(nbPts,0.1); % replace by the selected window: rectwin(nbPts); etc.
factzeropad = 4; % zero padding factor
nfftSLA = 2^nextpow2(nbPts)*factzeropad; % zero padding necessary to better estimate the slope
freqSLA = linspace(0,Fe/2,nfftSLA/2+1); % frequency vector in km-1
freqSLA_norm = linspace(0,0.5,nfftSLA/2+1); % normalised frequency vector

% Spectral Analysis by ARWARP (see IGARSS paper)
b = 0.9; % warping parameter - recommanded: 0.9
P = 5; % ARWARP model order - recommanded: 5

disp('Spectral analysis parameters');
disp(['Periodogram with Tukey (10%) window and zero-padding by a factor ' num2str(factzeropad)]);
disp(['ARWARP with warping parameter = ' num2str(b) ' and model order = ' num2str(P)]);

% parameters for noise floor estimation
freqplateaubruit_norm = [0.25   0.5]; % noise floor is estimated in this interval (normalised frequencies)

% parameters for slope estimation
% for linear regression
% The user can choose by inspection of the mean PSD
% Fourier based spectral estimation WITH zero-padding

SpecTF = zeros(nbSeg,nfftSLA);
for k = 1:nbSeg, % Spectral estimation and slope estimation is done on each segment
    x = zeros(nfftSLA,1);
    x = vSLA((k-1)*nbPts +1 : (k-1)*nbPts + nbPts)';
    x = detrend(x); % 
    
    % Spectral Analysis by Periodogram
    SpecTF(k,:) = abs(fft(x.*fen,nfftSLA)).^2 ./nbPts; % periodogramme
    % Correction  : to estimate the noise floor
    % Sigma of the noise = sqrt (K/(2*dist)) K being the level of noise
    % floor directly read on the spectrum
    SpecTF(k,:) = SpecTF(k,:) ./ (Fe/2);
end
PeriodoMoy = mean(SpecTF);

% if you want to choose the frequency interval on which linear regression
% is done, the lines between "LINEAR REG CHOICE" must be un-commented
% otherwise, recommended choice is
fkminkm = 90; % in km for real signals (IGARSS paper)
fkmaxkm = 200; % in km
% for simulated SLA, choice can be larger : [0.002 0.007 ] in normalized
% frequencies corresponding to 45-160 km (IGARSS paper)

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%% LINEAR REG CHOICE
    % figure('Name','Mean Periodogram');
    % h = axes; 
    % loglog(1./freqSLA_norm(2:end)/Fe,PeriodoMoy(2:nfftSLA/2+1)); hold on;
    % xlabel('km');
    % title('Mean PSD, with zero-padding');
    % set(h,'XDir','reverse');
    % 
    % disp('Choice of wavelength interval for slope estimation via linear regression');
    % disp('------------------------------------------------------------------------');
    % disp('Linear regression is done between');
    % fkminkm = input('          Wavelength 1 in km: ');
    % disp('and');
    % fkmaxkm = input('          Wavelength 2 in km: ');
    % v = axis;
    % loglog([fkminkm fkminkm],[v(3) v(4)],'g--');
    % loglog([fkmaxkm fkmaxkm],[v(3) v(4)],'g--');
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%% LINEAR REG CHOICE

% translation in normalized frequencies
fkmin = 1 / fkminkm / Fe;
fkmax = 1 / fkmaxkm / Fe;
freqpente_norm = sort([fkmin fkmax]);
% disp(['Linear regression is done between ' num2str(1/(freqpente_norm(2)*Fe)) ' km and ' num2str(1/(freqpente_norm(1)*Fe)) ' km']);
[pentespectre_moy, sigmabruit_moy,Sth_moy] = estimePenteBruit(PeriodoMoy(1:nfftSLA/2+1),freqSLA_norm,freqpente_norm,freqplateaubruit_norm);
disp(['Slope estimated on mean periodogram by linear regression with zero-padding: ' num2str(pentespectre_moy)]);

% for model fitting (see paper)
freq_begin_pente_norm = 0.001; 
freqLimits_norm = [0.0005 0.3]; % model constant-slope-noise floor is estimated in this interval (normalised frequencies)
disp(['Model fitting is done between ' num2str(1/(freqLimits_norm(2)*Fe)) ' km and ' num2str(1/(freqLimits_norm(1)*Fe)) ' km']);

% vector initialisations
SpecARWARP = zeros(nbSeg,nfftSLA/2+1);
alphaTFreg = zeros(nbSeg,1);
alphaTFfit = zeros(nbSeg,1);
alphaARreg = zeros(nbSeg,1);
alphaARfit = zeros(nbSeg,1);
sigmabruit = zeros(nbSeg,1);


for k = 1:nbSeg, % Spectral estimation and slope estimation is done on each segment
    x = zeros(nfftSLA,1);
    x = vSLA((k-1)*nbPts +1 : (k-1)*nbPts + nbPts)';
    x = detrend(x); % 
    
    % Spectral Analysis by Periodogram (done above)   
    SpectreSLA = SpecTF(k,1:nfftSLA/2+1);
    % Slope and noise floor estimation by Linear Regression on periodogram
    [pentespectre_reglin, sigmabruit_reglin,Sth_reglin] = estimePenteBruit(SpectreSLA,freqSLA_norm,freqpente_norm,freqplateaubruit_norm);
    sigmabruit(k) = sigmabruit_reglin; % best way to estimate
    alphaTFreg(k) = pentespectre_reglin;

    % Slope estimation by Model Fitting on periodogram (see paper)
    % f1 assumed to be known, fitting of the model on a specified frequency
    % band on periodogram
    [pentespectre_fitting, sigmabruit_fitting,Sth_fitting,theta_fit] = ...
        estime_noise_gamma(SpectreSLA,freqSLA_norm,freqLimits_norm,freq_begin_pente_norm,freqpente_norm); 
    alphaTFfit(k) = pentespectre_fitting;
    
    
    % Spectral Analysis by ARWARP
     disp(['Processing segment number ' num2str(k) '/' num2str(nbSeg) ' ... ']);
    SpecARWARP(k,:) = calculARWARP(x,P,b,freqSLA_norm);
    % Correction  : to estimate the noise floor
    % Sigma of the noise = sqrt (K/(2*dist)) K being the level of noise
    % floor directly read on the spectrum
    SpecARWARP(k,:) = SpecARWARP(k,:) ./ (Fe/2);
    
   
    SpectreSLA = SpecARWARP(k,:);
    % Slope estimation by Linear Regression on ARWARP
    [pentespectre_reglinW, sigmabruit_reglinW,Sth_reglinW] = estimePenteBruit(SpectreSLA,freqSLA_norm,freqpente_norm,freqplateaubruit_norm);
    alphaARreg(k) = pentespectre_reglinW;
    
    % Slope estimation by Model Fitting on ARWARP (see paper)
    % f1 assumed to be known, fitting of the model on a specified frequency
    % band on periodogram
    [pentespectre_fittingW, sigmabruit_fittingW,Sth_fittingW,theta_fitW] = ...
        estime_noise_gamma(SpectreSLA,freqSLA_norm,freqLimits_norm,freq_begin_pente_norm,freqpente_norm);
    alphaARfit(k) = pentespectre_fittingW;
    
    
end



%%%%%% SLOPE ESIMATION ON SUCCESSIVE SEGMENTS OF THE SLA FILE
figure('Name','Slope estimation along the SLA segments');
MK = 10;
plot(alphaTFreg,'b-*','MarkerSize',MK);hold on;
plot(alphaTFfit,'b-d','MarkerSize',MK);
plot(alphaARreg,'r-*','MarkerSize',MK);
plot(alphaARfit,'r-d','MarkerSize',MK);
v = axis;
plot([v(1) v(2)], [pentespectre_moy pentespectre_moy],'g --','LineWidth',3);
Leg = legend(['LR on Periodogram: Mean = ' num2str(mean(alphaTFreg),'%.2f') ' +/- ' num2str(2*std(alphaTFreg),'%.2f') ],...
    ['MF on Periodogram: Mean = ' num2str(mean(alphaTFfit),'%.2f') ' +/- ' num2str(2*std(alphaTFfit),'%.2f') ],...
    ['LR on ARWARP(' num2str(P) '): Mean = ' num2str(mean(alphaARreg),'%.2f') ' +/- ' num2str(2*std(alphaARreg),'%.2f') ],...
    ['MF on ARWARP(' num2str(P) '): Mean = ' num2str(mean(alphaARfit),'%.2f') ' +/- ' num2str(2*std(alphaARfit),'%.2f') ],...
    ['Welch: estimated slope = ' num2str(pentespectre_moy)]);
if simuSLA==1,
    plot([v(1) v(2)], [Simu.Slope Simu.Slope],'k --','LineWidth',2);
end

Leg.FontSize = 14;

xlabel('SLA segment number');
ylabel('Slope estimate');
if simuSLA == 1,
    title({['Slope estimation along the segments of ' nom ' with mean value +/- 2 \sigma' ] ; [ 'Confidence interval with CRB: ' num2str(alpha) ' +/- ' num2str(2*sqrt(CRB)) ]});
else
    title(['Slope estimation along the segments of ' nom ' with mean value +/- 2 \sigma' ] );
end
    


%%%%%% DISPLAY OF THE PSD OF THE SUCCESSIVE SEGMENTS OF THE SLA FILE
disp('Display of the PSD of');
disp('   0 - selected segments');
disp(['   1 - all segments (' num2str(nbSeg) ' figures)']);
disp(['   2 - End of programm ' ]);
rep = input('Your answer: ');

fk = sort([fkmin fkmax]);
if rep == 1,
    for numero = 1 : nbSeg,
        figure('Name',['PSD of segment number ' num2str(numero) ]);
        loglog(freqSLA,SpecTF(numero,1:nfftSLA/2+1),'b'); hold on;
        loglog(freqSLA,SpecARWARP(numero,1:nfftSLA/2+1),'r');
        xlabel('Wavenumber (cpkm)');ylabel('PSD (m2/cpkm)');
        v = axis;
        v(1) = freqSLA(2); v(2) = freqSLA(end); axis(v)
        loglog([fkmin*Fe fkmin*Fe],[v(3) v(4)],'g--');
        loglog([fkmax*Fe fkmax*Fe],[v(3) v(4)],'g--');
        
        deb = SpecARWARP(2);
        loglog(fk*Fe,[deb deb*(fk(2)/fk(1))^(-alphaARreg(numero))],'r','LineWidth',3);
        text(fk(2)*Fe, deb*(fk(2)/fk(1))^(-alphaARreg(numero)),['\alpha = ' num2str(alphaARreg(numero)) ]);
        deb = SpecTF(2);
        loglog(fk*Fe,[deb deb*(fk(2)/fk(1))^(-alphaTFfit(numero))],'b','LineWidth',3);
        text(fk(2)*Fe, deb*(fk(2)/fk(1))^(-alphaTFfit(numero)),['\alpha = ' num2str(alphaTFfit(numero)) ]);
        title(['SLA segment ' num2str(numero) ' of ' nom 'and esimated slopes: MF periodogram and LR ARWARP' ]);
    end
    
else

    while rep ==0,
    numero = input('Segment number for which display of the PSD is waited: ');

    figure('Name',['PSD of segment number ' num2str(numero) ]);
    loglog(freqSLA,SpecTF(numero,1:nfftSLA/2+1),'b'); hold on;
    loglog(freqSLA,SpecARWARP(numero,1:nfftSLA/2+1),'r');
    xlabel('Wavenumber (cpkm)');ylabel('PSD (m2/cpkm)');
    v = axis;
    v(1) = freqSLA(2); v(2) = freqSLA(end); axis(v)
    loglog([fkmin*Fe fkmin*Fe],[v(3) v(4)],'g--');
    loglog([fkmax*Fe fkmax*Fe],[v(3) v(4)],'g--');
   
    deb = SpecARWARP(2);
        loglog(fk*Fe,[deb deb*(fk(2)/fk(1))^(-alphaARreg(numero))],'r','LineWidth',3);
        text(fk(2)*Fe, deb*(fk(2)/fk(1))^(-alphaARreg(numero)),['\alpha = ' num2str(alphaARreg(numero)) ]);
        deb = SpecTF(2);
        loglog(fk*Fe,[deb deb*(fk(2)/fk(1))^(-alphaTFfit(numero))],'b','LineWidth',3);
        text(fk(2)*Fe, deb*(fk(2)/fk(1))^(-alphaTFfit(numero)),['\alpha = ' num2str(alphaTFfit(numero)) ]);
        title(['SLA segment ' num2str(numero) ' of ' nom 'and esimated slopes: MF periodogram and LR ARWARP' ]);

    rep = input('One more PSD display of another segment ? (yes = 0): ');

    end
end


